/*
 * Created on 26-Dec-2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.blazinggames.cq1.cqcs;

// Java imports 
import java.awt.*;
import java.awt.event.*;
import java.io.*;

//import java.util.*;

// Blazing Games Imports
import com.blazinggames.util.conset.*;

/**
 * Coffee Quest Construction Set
 * Copyright (C) 2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Billy D. Spelchan
 * @version 0.00
 */
public class CQCS 
extends Frame
implements ActionListener, AdjustmentListener, MouseListener, MouseMotionListener
{
// *********************
// ***** CONSTANTS *****
// *********************

	private static final String FILE_NEW = "New Map";
	private static final String NEW_16 = "Small (16x16)";
	private static final String NEW_32 = "Medium (32x32)";
	private static final String NEW_64 = "Large (64x64)";
	private static final String NEW_128 = "Huge (128x128)";
	private static final String HELP_ABOUT = "About";
	
	private static String BASE64ENC = "-+abcdefghijklmnopqrstuHIJKLMNOPQRSTUvwxyz13VWXYZ57ABCDEFG8624@*";
    
	// *** VARIABLES ***
	TilemapViewer _mapviewer;
	GridButton _modeGrid, _tileGrid;
	Image _tileImage, _modeImage;
	Tilemap _map;
	Scrollbar _vbar, _hbar;
	Label _info;
	
	FileDialog fdSave, fdOpen;
	Layout _layout;
	/**
		The constructor class.  Adjusts the windows size and title, creates
		the menu bar, then enables the close window event.
	*/
	public CQCS()
	{
		// Make menu bar.
		initMenus();

		// Set up the window
		setTitle("Coffee Quest construction Set");
		setResizable(true);
		initLayout();
		// Make it possible to close the window
		enableEvents(WindowEvent.WINDOW_CLOSING);
		pack();
	}

	private void initLayout()
	{
		Toolkit tk = Toolkit.getDefaultToolkit();
		
		_tileImage = tk.getImage("cqtiles.gif");
		_modeImage = tk.getImage("cqcs.gif");
		_map = new Tilemap(32, 32);
		_map.clear(0);
		_modeGrid = new GridButton(_modeImage, 64, 64, 1, 1);
		_modeGrid.setButtonsPerRow(1);
		_tileGrid = new GridButton(_tileImage, 16, 16, 16, 1);
		_tileGrid.setButtonsPerRow(4);
		_mapviewer = new TilemapViewer(_map, _tileImage, 16, 16);
		_mapviewer.addMouseListener(this);
		_mapviewer.addMouseMotionListener(this);
		_layout = new Layout();
		_hbar = new Scrollbar(Scrollbar.HORIZONTAL, 0,1, 0,31);
		_hbar.addAdjustmentListener(this);
		_vbar = new Scrollbar(Scrollbar.VERTICAL, 0,1, 0,31);
		_vbar.addAdjustmentListener(this);
		_info = new Label("Info");
		setLayout(_layout);
		add(Layout.VIEWER, _mapviewer);
		add(Layout.VIEWER_HBAR, _hbar);
		add(Layout.VIEWER_VBAR, _vbar);
		add(Layout.MODE, _modeGrid);
		add(Layout.TILE, _tileGrid);
		add(Layout.INFO, _info);

//		System.out.println("initLayout finished");
	}
    
	/*
		Create the menus for the window
	 */
	private void initMenus() {
		MenuBar mb = new MenuBar();
		// FILE menu
		Menu m = new Menu("File");
		Menu mn = new Menu(FILE_NEW);
		MenuItem mi = new MenuItem(NEW_16);
		mi.addActionListener(this);
		mn.add(mi);
		mi = new MenuItem(NEW_32);
		mi.addActionListener(this);
		mn.add(mi);
		mi = new MenuItem(NEW_64);
		mi.addActionListener(this);
		mn.add(mi);
		mi = new MenuItem(NEW_128);
		mi.addActionListener(this);
		mn.add(mi);
		m.add(mn);
		mi = new MenuItem("Open...");
		mi.addActionListener(this);
		m.add(mi);
		mi = new MenuItem("Save");
		mi.addActionListener(this);
		m.add(mi);
		mi = new MenuItem("Save As...");
		mi.addActionListener(this);
		m.add(mi);
		mi = new MenuItem("-");
		m.add(mi);
		mi = new MenuItem("Quit");
		mi.addActionListener(this);
		m.add(mi);
		mb.add( m );

		// EDIT menu
/*
		m = new Menu("Edit");
		mi = new MenuItem("Cut");
		mi.addActionListener(this);
		m.add(mi);
		mi = new MenuItem("Copy");
		mi.addActionListener(this);
		m.add(mi);
		mi = new MenuItem("Paste");
		mi.addActionListener(this);
		m.add(mi);
		mb.add( m );
*/
		// ABOUT Menu
		m = new Menu("Help");
		mi = new MenuItem(HELP_ABOUT);
		mi.addActionListener(this);
		m.add(mi);
		mb.add( m );
        
		setMenuBar( mb );
	}


	private void menu_FileNew(int size) {
		System.out.println("File->new called");
		_map.resize(size, size);
		_map.clear(0);
		Dimension ds = _mapviewer.getTileDisplaySize();
		Dimension cs = _mapviewer.getSize();
		_vbar.setMaximum(size);
		_hbar.setMaximum(size);
		_mapviewer.setPosition(new Point(0,0));
		_vbar.setValue(0);
		_hbar.setValue(0);
		_vbar.setVisibleAmount(cs.height / Math.max(1,ds.height));
		_hbar.setVisibleAmount(cs.width / Math.max(1,ds.width));
		repaint();
		_mapviewer.repaint();
		fdSave = null;
	}

	private void menu_FileOpen() {
		System.out.println("File->Open");
		if (fdOpen == null)
		{
			fdOpen = new FileDialog(this, "Open Map", FileDialog.LOAD);
		}
		fdOpen.show();
		if (fdOpen.getFile() != null) {
			System.out.println("load file: " + fdOpen.getDirectory() + fdOpen.getFile());
			doLoad(fdOpen.getDirectory() + fdOpen.getFile());
		}
	}

	private void menu_FileSave() {
		if (fdSave == null) {
			menu_FileSaveAs();
			return;
		}
		if (fdSave.getFile() == null) {
			menu_FileSaveAs();
			return;
		}
		System.out.println("Save file: " + fdSave.getDirectory() + fdSave.getFile());
		doSave(fdSave.getDirectory() + fdSave.getFile());
	}

	private void menu_FileSaveAs() {
		if (fdSave == null)
			fdSave = new FileDialog(this, "Save map as", FileDialog.SAVE);
		fdSave.show();
		if (fdSave.getFile() != null) {
			System.out.println("Save file as: " + fdSave.getDirectory() + fdSave.getFile());
			doSave(fdSave.getDirectory() + fdSave.getFile());
		}
	}

	private void menu_HelpAbout()
	{
	}
	
	/**
		Handle menu events (well, the quit option) and Panel actions.
	*/
	public void actionPerformed(ActionEvent e)
	{
		String sID = e.getActionCommand();

		// If selected menu is Quit
		if (sID.equals(NEW_16) )
			menu_FileNew(16);
		else if (sID.equals(NEW_32) )
			menu_FileNew(32);
		else if (sID.equals(NEW_64) )
			menu_FileNew(64);
		else if (sID.equals(NEW_128) )
			menu_FileNew(128);
		else if ( sID.equals( "Open..." ) )
			menu_FileOpen();
		else if ( sID.equals( "Save" ) )
			menu_FileSave();
		else if ( sID.equals( "Save As..." ) )
			menu_FileSaveAs();
		else if ( sID.equals( "Quit" ) )
		{
			setVisible(false);
			dispose();
			System.exit(0);
		}
		else if ( sID.equals(HELP_ABOUT))
			menu_HelpAbout();
	}


	/**
		Handle closing of the window
	*/
	protected void processEvent(AWTEvent e) {
		// See of window being closed
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			setVisible(false);
			dispose();
			System.exit(0);
		}
		else if (e.getID() == WindowEvent.COMPONENT_RESIZED)
		{
			Dimension ds = _mapviewer.getTileDisplaySize();
			Dimension cs = _mapviewer.getSize();
			Point p = _mapviewer.getPosition();
			_vbar.setVisibleAmount(cs.height / Math.max(1,ds.height));
			_hbar.setVisibleAmount(cs.width / Math.max(1,ds.width));
			System.out.println("Window resized");
		}
		// if not let java handle the event
		super.processEvent(e);
	}


// *********************************************
// ***** METHODS - From AdjustmentListener *****
// *********************************************

	public void adjustmentValueChanged(AdjustmentEvent e)
	{
		int topx = _hbar.getValue();
		int topy = _vbar.getValue();
		_mapviewer.setPosition(new Point(topx, topy));
		_info.setText("Top X = " + topx + ", Top Y = " + topy);
//		System.out.println(_info.getText());
		repaint();
		_mapviewer.repaint();
	}
	
// ****************************************
// ***** METHODS - From MouseListener *****
// ****************************************

	/* (non-Javadoc)
		* @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
		*/
	public void mouseClicked(MouseEvent arg0) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
		* @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
		*/
	public void mousePressed(MouseEvent e)
	{
		Point p = _mapviewer.getPosition();
		int x = e.getX() / _mapviewer.getTileDisplaySize().width + p.x;
		int y = e.getY() / _mapviewer.getTileDisplaySize().height + p.y;
//		System.out.println("Mouse clicked at " + x + ", " + y);
		if (_modeGrid.getSelected() == 0)
			_map.setTile(x, y, _tileGrid.getSelected());
		_mapviewer.repaint();
	}

	/* (non-Javadoc)
		* @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
		*/
	public void mouseReleased(MouseEvent e) 
	{
	}

	/* (non-Javadoc)
		* @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
		*/
	public void mouseEntered(MouseEvent arg0) 
	{
	}

	/* (non-Javadoc)
		* @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
		*/
	public void mouseExited(MouseEvent arg0) 
	{
	}

// **********************************************
// ***** METHODS - From MouseMotionListener *****
// **********************************************

	public void mouseDragged(MouseEvent e)
	{
		Point p = _mapviewer.getPosition();
		int x = e.getX() / _mapviewer.getTileDisplaySize().width + p.x;
		int y = e.getY() / _mapviewer.getTileDisplaySize().height + p.y;
//		System.out.println("Mouse dragged at " + x + ", " + y);
		if (_map.getTile(x, y) != _tileGrid.getSelected())
		{
			if (_modeGrid.getSelected() == 0)
				_map.setTile(x, y, _tileGrid.getSelected());
			_mapviewer.repaint();
		}
	}

	public void mouseMoved(MouseEvent e)
	{
        
	}

// ***** METHODS - saving/loading *****

	public void doSave(String fname)
	{
		int cntrX, cntrY, tindex, msize;

		try
		{
			// Open file
			FileWriter fos = new FileWriter(fname);
			fos.write("CQ1\n");
			msize = _map.getWidth();
			fos.write(msize+"\n");
			for (cntrY = 0; cntrY < msize; ++cntrY)
			{
				for (cntrX = 0; cntrX < msize; ++cntrX)
				{
					fos.write(BASE64ENC.charAt(_map.getTile(cntrX, cntrY)));
				}
				fos.write("\n");
			}
			fos.flush();
			System.out.println("File should have been writen");
			// Close file
			fos.close();
		}
		catch (IOException e) {
			System.out.println("IO Error");
		}
        
	}

	public void doLoad(String fname)
	{
		int cntrX, cntrY, tindex, msize;
		String s;
		
		try
		{
			// Open file
			BufferedReader fis = new BufferedReader(new FileReader(fname));
			s = fis.readLine();
			if (s.indexOf("CQ1") >= 0)
			{
				// read map size then set and clear (in case file problem) map 
				msize = Integer.parseInt(fis.readLine());
				menu_FileNew(msize);
				for (cntrY = 0; cntrY < msize; ++cntrY)
				{
					s = fis.readLine();
					for (cntrX = 0; cntrX < msize; ++cntrX)
					{
						tindex = s.charAt(cntrX);
						_map.setTile(cntrX, cntrY, BASE64ENC.indexOf(tindex));
					}
				}
			}
			// Close file
			fis.close();
			_mapviewer.repaint();
		}
		catch (IOException e) {
			System.out.println("IO Error");
		}
        
	}
	
	public static void main (String args[]) {
		// Create an instance of the mainframe
		CQCS mainFrame = new CQCS();
		// Now show it.
		mainFrame.show();
	}

}
