/*
 * Created on 26-Dec-2003
 *
 * To change this generated comment go to 
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package com.blazinggames.cq1.cqcs;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.awt.Insets;

/**
 * Coffee Quest Construction Set
 * Copyright (C) 2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Billy D. Spelchan
 * @version 0.00
 */
public class Layout 
implements LayoutManager 
{
// *********************
// ***** CONSTANTS *****
// *********************

	public static final String VIEWER = "VIEWER";
	public static final String VIEWER_HBAR = "HBAR";
	public static final String VIEWER_VBAR = "VBAR";
	public static final String MODE = "MODE";
	public static final String TILE = "TILE";
	public static final String INFO = "INFO";
	
// *********************
// ***** VARIABLES *****
// *********************

	protected Component _viewer;
	protected Component _viewerHBar;
	protected Component _viewerVBar;
	protected Component _mode;
	protected Component _tile;
	protected Component _info;

// ***********************	
// ***** CONSTRUCTOR *****
// ***********************

	public Layout()
	{
		_viewer = null;
		_viewerHBar = null;
		_viewerVBar = null;
		_mode = null;
		_tile = null;
		_info = null;		
	}

	/* (non-Javadoc)
	 * @see java.awt.LayoutManager#addLayoutComponent(java.lang.String, java.awt.Component)
	 */
	public void addLayoutComponent(String s, Component obj) 
	{
		if (VIEWER.equalsIgnoreCase(s))
			_viewer = obj;
		else if (VIEWER_HBAR.equalsIgnoreCase(s))
			_viewerHBar = obj;
		else if (VIEWER_VBAR.equalsIgnoreCase(s))
			_viewerVBar = obj;
		else if (MODE.equalsIgnoreCase(s))
			_mode = obj;
		else if (TILE.equalsIgnoreCase(s))
			_tile = obj;
		else if (INFO.equalsIgnoreCase(s))
			_info = obj;		
		
	}

	/* (non-Javadoc)
	 * @see java.awt.LayoutManager#removeLayoutComponent(java.awt.Component)
	 */
	public void removeLayoutComponent(Component c) 
	{
		if (_viewer == c)
			_viewer = null;
		else if (_viewerHBar == c)
			_viewerHBar = null;
		else if (_viewerVBar == c)
			_viewerVBar = null;
		else if (_mode == c)
			_mode = null;
		else if (_tile == c)
			_tile = null;
		else if (_info == c)
			_info = null;		
	}

	/* (non-Javadoc)
	 * @see java.awt.LayoutManager#preferredLayoutSize(java.awt.Container)
	 */
	public Dimension preferredLayoutSize(Container c) 
	{
		int pwidth = 0;
		int pheight = 0;
		Dimension d;
		int cmdW = 0;
		int cmdH = 0;
		if (_viewer != null)
		{
			d = _viewer.getPreferredSize();
			pwidth +=  d.getWidth();
			pheight += d.getHeight();
		}
		if (_viewerHBar != null)
		{
			d = _viewerHBar.getPreferredSize();
			pheight += d.getHeight();
		}
		if (_viewerVBar != null)
		{
			d = _viewerVBar.getPreferredSize();
			pwidth +=  d.getWidth();
		}
		
		if (_mode != null)
		{
			d = _mode.getPreferredSize();
			cmdW = d.width;
			cmdH = d.height;
		}
		if (_tile != null)
		{
			d = _tile.getPreferredSize();
			cmdW = Math.max(cmdW, d.width);
			cmdH += d.height;
		}
		
		pheight = Math.max(pheight, cmdH);
		
		if (_info != null)
		{
			d = _info.getPreferredSize();
			pheight += d.height;
		}
		
		Insets i = c.getInsets();
		return new Dimension(i.left + i.right + pwidth, i.top + i.bottom + pheight);
	}

	/* (non-Javadoc)
	 * @see java.awt.LayoutManager#minimumLayoutSize(java.awt.Container)
	 */
	public Dimension minimumLayoutSize(Container arg0) 
	{
		return new Dimension(640, 480);
	}

	/* (non-Javadoc)
	 * @see java.awt.LayoutManager#layoutContainer(java.awt.Container)
	 */
	public void layoutContainer(Container c) 
	{
		Insets i = c.getInsets();
		int maxWidth = c.getWidth() - i.left - i.right;
		int maxHeight = c.getHeight() - i.top - i.bottom;
		int hbHeight = 0;
		int vbWidth = 0;
		int rightHeight = 10;
		int rightWidth = 0;
		int infoHeight = 0;
		Dimension temp;
		Dimension modeD = null, tileD = null;
        
		// calculate sizes
		if (_info != null)
			infoHeight = _info.getPreferredSize().height;
		if (_mode != null)
		{
			modeD = _mode.getPreferredSize();
			rightWidth = modeD.width;
			rightHeight += modeD.height;
		}
		if (_tile != null)
		{
			tileD = _tile.getPreferredSize();
			rightWidth = Math.max(rightWidth, tileD.width);
			rightHeight += tileD.height;
//			System.out.println("Tile width =" +tileD.width+","+tileD.height);
		}
		if ((infoHeight + rightHeight) > maxHeight)
			infoHeight = maxHeight - rightHeight;

		if (_viewerHBar != null)
		{
			temp = _viewerHBar.getPreferredSize();
			hbHeight = temp.height;
		}
		if (_viewerVBar != null)
		{
			temp = _viewerVBar.getPreferredSize();
			vbWidth = temp.width;
		}

		// layout
		int viewerW = maxWidth - vbWidth - rightWidth - 10;
		int viewerH = maxHeight - hbHeight - infoHeight;
		int rightStart = maxWidth - rightWidth - 10 + i.left;
//		System.out.println("Width = " + maxWidth + " Right = " + rightStart);
		if (_viewer != null)
			_viewer.setBounds(i.left,i.top, viewerW, viewerH);
		if (_viewerHBar != null)
			_viewerHBar.setBounds(i.left, viewerH+i.top, viewerW, hbHeight);
		if (_viewerVBar != null)
			_viewerVBar.setBounds(i.left+viewerW,i.top,vbWidth, viewerH);
		if (_mode != null)
			_mode.setBounds(rightStart,i.top, modeD.width+10, modeD.height+10);
		if (_tile != null)
			_tile.setBounds(rightStart, i.top+modeD.height+10, tileD.width+10, tileD.height+10);
		if (_info != null)
			_info.setBounds(0, maxHeight-infoHeight, maxWidth,infoHeight);
	}
}

